"""
Legal Disclaimer.

(c) 2018 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at https://aws.amazon.com/agreement/ or other written
agreement between Customer and Amazon Web Services, Inc.


Security Control: RDS Aurora must use KMS CMK for Encryption at Rest
Description: Checks RDS Aurora instances during launch to ensure that CMK KMS keys are being used
for encryption.  Alerts if CMK is not in use.

Runtime: Python 3.6
Last Modified: 05/16/2018
"""

import os
import boto3
from botocore.exceptions import ClientError
import logging

OUTBOUND_TOPIC_ARN = os.environ["outbound_topic_arn"]


def lambda_handler(event, context):
    """
    Main Function.

    Will call and return the value of the default service key which will later on be used to compare
    to the KMS Key used by the Aurora RDS Instance.

    If the Aurora RDS Instance is in violation, it will call the SNS function.
    """
    setup_logging()
    log.info(event)

    check_rds_engine_type = engine_check(event)
    if check_rds_engine_type:
        default_alias = get_alias()
        violation_rds_instance = check_rds_instance(default_alias, event)

    if violation_rds_instance:
        subject = "Violation - Aurora RDS Instance Not Using a CMK"
        message = create_non_compliance_message(violation_rds_instance, event, context)
        send_violation(OUTBOUND_TOPIC_ARN, message, subject)

def engine_check(event):
    """Function that determines if the RDS Instance is an Aurora instance."""
    engine = event["detail"]["requestParameters"]["engine"]
    if engine == "aurora-postgresql" or engine == "aurora-mysql":
        return 1
    else:
        return 0

def check_rds_instance(default_alias, event):
    """
    Function that will check the RDS Instance.

    Compares the value of the Aurora RDS Service KMS key and what's used by the Aurora RDS Instance.


    If the Aurora RDS instance is using the default key, it will be appended to the list, where it's
    converted to a string.
    """
    violation_rds_instance = []
    try:
        if event["detail"]["responseElements"]["storageEncrypted"] is True:
                check_key = event["detail"]["responseElements"]["kmsKeyId"]
                check_key_id = check_key.split("/")
                checked_id = check_key_id[1]
                if checked_id == default_alias:
                    violation_rds_instance.append(event["detail"]["requestParameters"]["dBInstanceIdentifier"])
                else:
                    return 0
        elif event["detail"]["responseElements"]["storageEncrypted"] is False:
            violation_rds_instance.append(event["detail"]["requestParameters"]["dBInstanceIdentifier"])
        violation_rds_instance = ''.join(violation_rds_instance)
        return violation_rds_instance
    except KeyError as err:
        log.info(err)
    

def get_alias():
    """Function will retrieve the alias for the default service key for RDS."""
    client = boto3.client('kms')
    try:
        response = client.list_aliases()
    except ClientError as err:
        log.info(err)
        return False
    for alias in response["Aliases"]:
        if alias["AliasName"] == "alias/aws/rds":
            default_alias = alias["TargetKeyId"]
    return default_alias

def send_violation(outbound_topic_arn, message, subject):
    """Function will publish the AWS SNS notification detailing when a violation has occurred."""
    findsnsregion = outbound_topic_arn.split(":")
    snsregion = findsnsregion[3]
    sendclient = boto3.client('sns', region_name=snsregion)
    try:
        sendclient.publish(
            TopicArn=outbound_topic_arn,
            Message=message,
            Subject=subject
        )
    except ClientError as err:
        log.info(err)
        return False

def create_non_compliance_message(violation_rds_instance, event, context):
    """Function will structure the outgoing AWS SNS notification format."""
    message = "Violation - Aurora RDS Instance Not Using a CMK  \n\n"
    message += 'RDS Instance: ' + violation_rds_instance + '\n'
    message += 'Account: ' + event["account"] + "\n"
    message += "Region: " + event["detail"]["awsRegion"] + "\n"
    message += "\n\n"
    message += "This notification was generated by the Lambda function " + \
        context.invoked_function_arn
    return message

def setup_logging():
    """
    Logging Function.

    Creates a global log object and sets its level.
    """
    global log
    log = logging.getLogger()
    log_levels = {'INFO': 20, 'WARNING': 30, 'ERROR': 40}

    if 'logging_level' in os.environ:
        log_level = os.environ['logging_level'].upper()
        if log_level in log_levels:
            log.setLevel(log_levels[log_level])
        else:
            log.setLevel(log_levels['ERROR'])
            log.error("The logging_level environment variable is not set to INFO, WARNING, or \
                        ERROR.  The log level is set to ERROR")
    else:
        log.setLevel(log_levels['ERROR'])
    log.info('Logging setup complete - set to log level ' + str(log.getEffectiveLevel()))
